#!/bin/dash

### configuration
QEMU_AUDIO_DRV=${QEMU_AUDIO_DRV-alsa}
export QEMU_AUDIO_DRV
#CPUTYPE= # default is empty, can be "qemu64" (default) or "host" "kvm64" etc
CPUS=${CPUS:-1}
MEM=${MEM:-1024}
VGA_TYPE=${VGA_TYPE:-cirrus-vga} # or cirrus-vga (cirrus) or VGA (std)
#POINTER= # default is empty, can be usb-mouse, usb-tablet, or usb-wacom-tablet
MACH_TYPE=${MACH_TYPE:-pc}
BOOT_PARAM=${BOOT_PARAM:-basesfs=device:sda:/ pfix=nox} # video=800x600-24 or VGA-1:800x600-24
QEMU_UI=${QEMU_UI:-sdl}
HOSTDVD=${HOSTDVD:+-drive if=scsi,file=/dev/$HOSTDVD,format=raw}
FATDOG_INITRD=${FATDOG_INITRD:-../pkg/000/fatdog-initrd}
NIC=${NIC:-user,hostfwd=tcp::2222-:22}
UEFI_BIOS=${UEFI_BIOS:-uefi-bios.bin}
SOUNDHW=${SOUNDHW--device AC97}
NO_FLOPPY=${NO_FLOPPY--global isa-fdc.fdtypeA=none} # specify NO_FLOPPY= to enable floppy emulation
#SHARE=    # default is empty, but can be set to a path to share (read-only)
#SHARE_RW= # default is empty, but can be set to a path to share

#KVER= # default is empty
KERNEL=${KERNEL:-../kernels/vmlinuz}
MODULES=${MODULES:-../kernels/kernel-modules.sfs}
INITRD=${INITRD:-qemu-initrd}
BASE_SFS=${BASE_SFS:-fd64.sfs}

### helpers ###
run_in_my_dir() {
	mydir=$(readlink -f "$0")
	mydir=${mydir%/*}
	cd $mydir
}

die() {
	echo "$1"
	exit 
}

build_initrd() {
	local INITRD_TMP INITRD_OUT

	# easier way to set kernels than individual spec of KERNEL, MODULES and INITRD
	if [ "$KVER" ]; then
		KERNEL=${KERNEL}-$KVER
		MODULES=${MODULES}-$KVER
		INITRD=${INITRD}-$KVER
	fi

	# kernel or modules don't exist, bail out
	! [ -e $KERNEL ] && die "$KERNEL doesn't exist"
	! [ -e $MODULES ] && die "$MODULES doesn't exist"

	# initrd doesn't exist, make one
	if ! [ -e $INITRD ]; then
		INITRD_TMP=/tmp/initrd.$$ INITRD_OUT=$(readlink -f "$INITRD")
		cp -a "$FATDOG_INITRD" $INITRD_TMP
		cp $MODULES $INITRD_TMP/kernel-modules.sfs
		find $INITRD_TMP -name placeholder_for_fossil -delete
		( cd $INITRD_TMP; find . | cpio -o -H newc > $INITRD_OUT )
		rm -rf $INITRD_TMP
	fi
}

### main ###
run_in_my_dir
build_initrd
case $1 in
	uefi) UEFI="-bios $UEFI_BIOS"; shift ;;
esac
[ -e fd64-nls.sfs ]  && HDB="-drive file=fd64-nls.sfs,index=1,media=disk,format=raw"
[ -e fd64-devx.sfs ] && HDC="-drive file=fd64-devx.sfs,index=2,media=disk,format=raw"
[ -e disk.ext3 ]     && HDD="-drive file=disk.ext3,index=3,media=disk,format=raw"
[ -e disk2.img ]     && VDA1="-drive file=disk2.img,if=virtio,media=disk,format=raw"
[ "$HOSTDVD" ] && ! lsmod | grep -q ^sg && modprobe sg

# correct crazy mouse cursor under these two drivers
[ -z "$POINTER" ] &&
case $VGA_TYPE in
	virtio*|vmware-svga|qxl*) POINTER=usb-mouse ;;
esac
[ "$POINTER" ] && POINTER="-device $POINTER" # -usbdevice mouse/tablet/wacom-tablet for older qemu
[ "$CPUTYPE" ] && CPUTYPE="-cpu $CPUTYPE"
[ "$SHARE" ] && SHARE="-virtfs local,mount_tag=share,security_model=mapped,readonly=on,multidevs=remap,path=$SHARE"
[ "$SHARE_RW" ] && SHARE_RW="-virtfs local,mount_tag=share-rw,security_model=mapped,multidevs=remap,path=$SHARE_RW"

exec qemu-system-x86_64 -enable-kvm -machine $MACH_TYPE \
-display $QEMU_UI -m $MEM -device $VGA_TYPE $POINTER $CPUTYPE -smp $CPUS \
$UEFI -nic $NIC -usb $SOUNDHW \
-kernel $KERNEL -initrd $INITRD \
-append "$BOOT_PARAM $*" \
-drive file=$BASE_SFS,index=0,media=disk,format=raw \
$NO_FLOPPY $SHARE $SHARE_RW $HDB $HDC $HDD $VDA1 $HOSTDVD
